# Copyright (c) Stephan Martin <sm@sm-zone.net>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CA;

sub new {
   my $that = shift;
   my $self = {};

   my $class = ref($that) || $that;

   $self->{'init'} = shift;

   if(not -d $self->{'init'}->{'basedir'}) {
      print "create basedir: $self->{'init'}->{'basedir'}\n";
      mkdir($self->{'init'}->{'basedir'}, 0700);
      mkdir($self->{'init'}->{'basedir'}."/tmp", 0700);
   }

   opendir(DIR, $self->{'init'}->{'basedir'}) || do {
      print "error: can't open basedir: $!";
      exit(1);
   };

   $self->{'calist'} = [];

      while(my $ca = readdir(DIR)) { 
         chomp($ca);
         next if $ca eq ".";
         next if $ca eq "..";
         next if $ca eq "tmp";

         my $dir = $self->{'init'}->{'basedir'}."/".$ca;
         next unless -d $dir;
         push(@{$self->{'calist'}}, $ca);
         @{$self->{'calist'}} = sort(@{$self->{'calist'}});
         $self->{$ca}->{'dir'} = $dir;
         $self->{$ca}->{'cnf'} = $dir."/openssl.cnf";
      }
      closedir(DIR);

   bless($self, $class);
}

sub open_ca {
   my $self = shift;
   my $main  = shift;
   my $name = shift;

   my $ind;

   if((not defined($name)) || ($name eq "")) {
      $ind = $main->show_open_dialog(\$name);
      return if not defined $ind;
      $name = $self->{'calist'}->[$ind];
   }

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   # selected CA is already open
   return if ((defined($self->{'actca'})) &&
              ($name eq $self->{'actca'}));

   $self->{'actca'} = $name;

   $main->{'mw'}->configure(
         -title => "My Tiny CA Management $main->{'version'}".
                   " - $self->{'actca'}"
                   );

   delete($self->{'OpenSSL'});

   $main->init_openssl($name);

   if(defined($main->{'catab'})) {
      $main->{'catab'}->destroy();
      delete($main->{'catab'});
   }
   if(defined($main->{'certtab'})) {
      $main->{'certtab'}->destroy();
      delete($main->{'certtab'});
   }
   if(defined($main->{'reqtab'})) {
      $main->{'reqtab'}->destroy();
      delete($main->{'reqtab'});
   }
   if(defined($main->{'keytab'})) {
      $main->{'keytab'}->destroy();
      delete($main->{'keytab'});
   }

   $main->{'CERT'}->{'lastread'} = 0;
   $main->{'REQ'}->{'lastread'} = 0;
   $main->{'KEY'}->{'lastread'} = 0;

   $main->act_menu();
   $main->create_mframe();

   return;
}

sub delete_ca {
   my $self = shift;
   my $main = shift;

   my ($ind, @tmp, $name);

   $ind = $main->show_del_dialog(\$name);
   return if not defined $ind;
   $name = $self->{'calist'}->[$ind];

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   _rm_dir($self->{$name}->{'dir'});

   if((defined($self->{'actca'})) && 
      ($name eq $self->{'actca'})) {
     $self->{'actca'} = undef;
   }
   
   $main->{'catab'}->destroy();
   delete($main->{'catab'});

   $main->{'reqtab'}->destroy();
   delete($main->{'reqtab'});

   $main->{'keytab'}->destroy();
   delete($main->{'keytab'});

   $main->{'certtab'}->destroy();
   delete($main->{'certtab'});

   $main->{'nb'}->destroy();
   delete($main->{'nb'});

   $main->{'mframe'}->destroy();
   delete($main->{'mframe'});

   foreach(@{$self->{'calist'}}) {
      next if $_ eq $name;
      push(@tmp, $_);
   }
   $self->{'calist'} = \@tmp;

   $main->{'OpenSSL'} = undef;

   delete($self->{$name});

   $main->create_mframe();

   return;
}

sub create_ca {
   my $self      = shift;
   my $main      = shift;

   my ($opts, $name, $action, $index, $serial);
   $opts = {};
   $opts->{'days'} = 3650; # set default to 10 years

   $action = $main->show_ca_dialog($opts);

   return if $action eq "Cancel";

   while((not defined($opts->{'name'})) || 
         ($opts->{'name'} eq "") ||
         ($opts->{'name'} =~ /\s/)) {
      $main->print_warning("Name must be filled in and must"
                           ." not contain Spaces");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   };

   while((not defined($opts->{'passwd'}) || 
          not defined($opts->{'passwd2'})) ||
         $opts->{'passwd'} ne $opts->{'passwd2'}) {
      $main->print_warning("Passwords don't match");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   while(not defined($opts->{'c'}) ||
         length($opts->{'c'}) != 2) {
      $main->print_warning("Country must be exact 2 letter code");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   while((not defined($opts->{'c'})) ||
         ($opts->{'c'} eq "") ||
         (not defined($opts->{'st'})) ||
         ($opts->{'st'} eq "") ||
         (not defined($opts->{'l'})) ||
         ($opts->{'l'} eq "") ||
         (not defined($opts->{'o'})) ||
         ($opts->{'o'} eq "") ||
         (not defined($opts->{'ou'})) ||
         ($opts->{'ou'} eq "") ||
         (not defined($opts->{'cn'})) ||
         ($opts->{'cn'} eq "") ||
         (not defined($opts->{'email'})) ||
         ($opts->{'email'} eq "")) {
      $main->print_warning("Please fill in every field");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   $name = $opts->{'name'};

   if(defined($self->{$name})) {
      $main->print_warning("CA: $name already exists");
      return;
   }

   # create directories
   $self->{$name}->{'dir'} = $self->{'init'}->{'basedir'}."/".$name;

   mkdir($self->{$name}->{'dir'}, 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/req", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/keys", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/certs", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/crl", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/newcerts", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   # create configuration file
   my $in  = $self->{'templates'}."/openssl.cnf";
   my $out = $self->{$name}->{'dir'}."/openssl.cnf";

   open(IN, "<$in") || do {
      $main->print_error("can't open template: $!");
      return;
   };
   open(OUT, ">$out") || do {
      $main->print_error("can't open outfile: $!");
      return;
   };
   while(<IN>) {
      s/\%dir\%/$self->{$name}->{'dir'}/;
      print OUT;
   }
   close IN;
   close OUT;
   $self->{$name}->{'cnf'} = $out;

   # create some more files
   $index = $self->{$name}->{'dir'}."/index.txt";
   open(OUT, ">$index") || do {
      $main->print_error("can't open indexfile: $!");
      return;
   };
   close OUT;

   $serial = $self->{$name}->{'dir'}."/serial";
   open(OUT, ">$serial") || do {
      $main->print_error("can't open serialfile: $!");
      return;
   };
   print OUT "01";
   close OUT;

   # initial configuration
   $action = $main->show_config_dialog($name);

   if($action eq 'Cancel') {
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }
   
   # cleanup actual session
   $main->{'OpenSSL'} = undef;
   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($name);
   }

   # create CA certifikate
   $main->{'OpenSSL'}->genKey( 
         BITS      => 2048,
         OUTFILE   => $self->{$name}->{'dir'}."/cacert.key",
         PASSWD    => $opts->{'passwd'},
         ALGORITHM => 'des3'
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.key") {
      $main->print_error("genKey failed");
   }

   $main->{'OpenSSL'}->genReq( 
         OUTFILE => $self->{$name}->{'dir'}."/cacert.req",
         PASSWD  => $opts->{'passwd'},
         DN      => [ $opts->{'c'},
                      $opts->{'st'},
                      $opts->{'l'},
                      $opts->{'o'},
                      $opts->{'ou'},
                      $opts->{'cn'},
                      $opts->{'email'},
                      '',
                      ''
                    ],
         KEYFILE => $self->{$name}->{'dir'}."/cacert.key"
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.req") {
      unlink($self->{$name}->{'dir'}."/cacert.key");
      unlink($self->{$name}->{'dir'}."/cacert.req");
      $main->print_error("genReq failed");
   }

   $main->{'OpenSSL'}->genCert( 
         OUTFILE => $self->{$name}->{'dir'}."/cacert.pem",
         KEYFILE => $self->{$name}->{'dir'}."/cacert.key",
         REQFILE => $self->{$name}->{'dir'}."/cacert.req",
         PASSWD  => $opts->{'passwd'},
         DAYS    => $opts->{'days'}
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.pem") {
      unlink($self->{$name}->{'dir'}."/cacert.key");
      unlink($self->{$name}->{'dir'}."/cacert.req");
      unlink($self->{$name}->{'dir'}."/cacert.pem");
      $main->print_error("genCert failed");
   }

   unlink($self->{$name}->{'dir'}."/cacert.req");

   $main->{'OpenSSL'}->issueCrl(
         CAKEY    => $self->{$name}->{'dir'}."/cacert.key",
         CACAERT  => $self->{$name}->{'dir'}."/cacert.pem",
         PASSWD   => $opts->{'passwd'},
         DAYS     => 365,
         OUTFILE  => $self->{$name}->{'dir'}."/crl/crl.pem",
         OUTFORM  => "PEM"
         );

   if (not -s $self->{$name}->{'dir'}."/crl/crl.pem") {
      $main->print_error("issueCrl failed");
   }

   # seems to be done
   push(@{$self->{'calist'}}, $name);
   @{$self->{'calist'}} = sort(@{$self->{'calist'}});
   $main->print_info("CA: $name created");

   $self->open_ca($main, $name);
   return;
}

sub parse_ca {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $self->{'actca'};
   my $cacert = $self->{$ca}->{'dir'}."/cacert.pem";

   my($x509, $parsed);

   if(not defined $main->{'OpenSSL'}) {
      $main->init_openssl($ca);
   }

   $x509    = OpenCA::X509->new( 
      SHELL  => $main->{'OpenSSL'}, 
      INFILE => $cacert
      );

   $parsed = $x509->getParsed();

   $self->{$ca}->{'parsed'} = $parsed;

   $self->{$ca}->{'EXPDATE'} = _get_date(
         $self->{$ca}->{'parsed'}->{'NOTAFTER'}
         );
}

sub export_ca_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $format, $x509, $out, $keyfile);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";

   $action = $main->show_ca_export_dialog(
         \$outfile,
         \$format
         );

   return if $action eq "Cancel";
   return if(not defined($outfile));

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $x509 = OpenCA::X509->new(
         SHELL  => $main->{'OpenSSL'},
         INFILE => $certfile,
         FORMAT => "PEM"
         );

   if($format eq "PEM") {
      $out = $x509->getPEM();
   } elsif ($format eq "DER") {
      $out = $x509->getDER();
   } elsif ($format eq "TXT") {
      $out = $x509->getTXT();
   } else {
      $main->print_warning("Invalid Format");
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning("Can't open Outfile!");
      return;
   };

   print OUT $out;
   close OUT;
   
   $main->print_info("Certificate exported to: $outfile");

   return;
}

sub export_crl {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $passwd,
      $format, $x509, $out, $keyfile);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";
   $outfile = "/tmp/crl.pem";

   $action = $main->show_crl_export_dialog(
         \$outfile,
         \$format,
         \$passwd
         );

   return if $action eq "Cancel";

   while((not defined($outfile)) || 
         ($outfile eq "")) {
      $main->print_warning("No outfile selected");
      $action = $main->show_crl_export_dialog(
            \$outfile,
            \$format,
            \$passwd
            );
      return if $action eq "Cancel";
   };

   while((not defined($passwd)) || 
         ($passwd eq "")) {
      $main->print_warning("No Password given");
      $action = $main->show_crl_export_dialog(
            \$outfile,
            \$format,
            \$passwd
            );
      return if $action eq "Cancel";
   };

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $main->{'OpenSSL'}->issueCrl(
         CAKEY    => $self->{$ca}->{'dir'}."/cacert.key",
         CACAERT  => $self->{$ca}->{'dir'}."/cacert.pem",
         PASSWD   => $passwd,
         DAYS     => 365,
         OUTFILE  => $outfile,
         OUTFORM  => $format
         );

   if (not -s $outfile) {
      $main->print_error("issueCrl failed");
   }

   $main->print_info("Crl exported to: $outfile");

   return;
}

sub _get_date {
   my $string = shift;

   $string =~ s/  / /g;

   my @t1 = split(/ /, $string);
   my @t2 = split(/:/, $t1[2]);

      $t1[0] = _get_index($t1[0]);

   my $ret = Time::Local::timelocal($t2[2],$t2[1],$t2[0],$t1[1],$t1[0],$t1[3]);

   return($ret);
}

sub _get_index {
   my $m = shift;
   my @a = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

   for(my $i = 0; $a[$i]; $i++) {
      return $i if($a[$i] eq $m);
   }
}

sub _rm_dir {
   my $dir = shift;

   my $dirh;

   opendir($dirh, $dir);

   while(my $f = readdir($dirh)) {
      next if $f eq '.';
      next if $f eq '..';

      if(-d $dir."/".$f) {
         _rm_dir($dir."/".$f);
      } else {
         unlink($dir."/".$f);
      }
   }
   closedir(DIR);

   rmdir($dir);
   
   return(0);
}

1
