# Copyright (c) Stephan Martin <sm@sm-zone.net>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package REQ;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

# create new request and key
sub create_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};

   my($action, $reqname, $reqfile, $keyfile, $parsed, $opts);

   $opts->{'bits'} = 2048;

   if(not defined($main->{'CA'}->{$ca}->{'parsed'})) {
      $main->{'CA'}->parse_ca($main);
   }
   $parsed = $main->{'CA'}->{$ca}->{'parsed'};

   # set defaults
   if(defined $parsed->{'C'}) {
      $opts->{'c'} = $parsed->{'C'};
   }
   if(defined $parsed->{'ST'}) {
      $opts->{'st'} = $parsed->{'ST'};
   }
   if(defined $parsed->{'L'}) {
      $opts->{'l'} = $parsed->{'L'};
   }
   if(defined $parsed->{'O'}) {
      $opts->{'o'} = $parsed->{'O'};
   }
   if(defined $parsed->{'OU'}) {
      $opts->{'ou'} = $parsed->{'OU'}->[0];
   }

   $action = $main->show_req_dialog($opts);

   return if $action eq "Cancel";

   while((not defined($opts->{'passwd'}) ||
          not defined($opts->{'passwd2'})) ||
         $opts->{'passwd'} ne $opts->{'passwd2'}) { 
      $main->print_warning("Passwords don't match");
      $action = $main->show_req_dialog($opts);
      return if $action eq "Cancel";
   }

   $reqname = MIME::Base64::encode(
      $opts->{'cn'}.":".
      $opts->{'email'}.":".
      $opts->{'ou'}.":".
      $opts->{'o'}.":".
      $opts->{'l'}.":".
      $opts->{'st'}.":".
      $opts->{'c'}, 
      ''
      );

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";
   $keyfile = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$reqname.".pem";

   if(-s $reqfile || -s $keyfile) {
      $action = $main->show_req_overwrite_warning();
      return if($action eq 'Cancel');
   }

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }
         
   $main->{'OpenSSL'}->genKey(
         BITS      => $opts->{'bits'},
         OUTFILE   => $keyfile,
         ALGORITHM => 'des3',
         PASSWD    => $opts->{'passwd'}
         );

   if (not -s $keyfile) { 
      unlink($keyfile);
      $main->print_warning("genKey failed");
      return;
   }

   $main->{'OpenSSL'}->genReq(
         OUTFILE => $reqfile,
         KEYFILE => $keyfile,
         PASSWD  => $opts->{'passwd'},
         DN      => [ $opts->{'c'},
                      $opts->{'st'},
                      $opts->{'l'},
                      $opts->{'o'},
                      $opts->{'ou'},
                      $opts->{'cn'},
                      $opts->{'email'},
                      '',
                      ''
                     ],
         );

   if (not -s $reqfile) { 
      unlink($keyfile);
      unlink($reqfile);
      $main->print_warning("genReq failed");
      return;
   }

   $main->create_mframe();

   return;
}

sub del_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $action);

   if(not defined $ind) {
      $main->print_warning("no request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   $action = $main->show_req_del_confirm();

   return if($action eq 'Cancel');

   unlink($reqfile);

   $main->create_mframe();

   return;
}

sub show_req {
}

sub read_reqlist {
   my $self   = shift;
   my $main   = shift;
   my $ca     = $main->{'CA'}->{'actca'};
   my $reqdir = $main->{'CA'}->{$ca}->{'dir'}."/req";

   my ($f, $modt);

   my $reqlist = [];

   $modt = (stat($reqdir))[9];

   if(defined($self->{'lastread'}) &&
      $self->{'lastread'} >= $modt) {  
      return(0);
   }

   opendir(DIR, $reqdir) || do {
      $main->print_warning("can't open reqdir");
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;
      $f =~ s/\.pem//;
      push(@{$reqlist}, MIME::Base64::decode($f));
   }
   @{$reqlist} = sort(@{$reqlist});
   closedir(DIR);

   $self->{'reqlist'} = $reqlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub sign_req {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'reqtab'}->{'list'}->info('selection');

   my($reqname, $req, $reqfile, $serial, $certout, $certfile,
      $passwd, $action, $certtype, $days, $time);

   $time = time();
   
   if(not defined $ind) {
      $main->print_warning("no request selected");
      return;
   }

   $reqname = $self->{'reqlist'}->[$ind];

   $req = MIME::Base64::encode($reqname, '');

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$req.".pem";

   if(not -s $reqfile) {
      $main->print_warning("Request not found");
      return;
   }

   $action = $main->show_req_sign_dialog(
         \$passwd,
         \$days,
         \$certtype
         );

   return if($action eq 'Cancel');

   if(($time + ($days * 86400)) > $main->{'CA'}->{$ca}->{'EXPDATE'}) {
      $action = $main->show_req_date_warning;
      return if($action eq 'Cancel');
   }

   $serial = $main->{'CA'}->{$ca}->{'dir'}."/serial";
   open(IN, "<$serial") || do {
      $main->print_warning("can't read serial");
      return;
   };
   $serial = <IN>;
   chomp($serial);
   close IN;

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $main->{'OpenSSL'}->issueCert(
         REQFILE     => $reqfile,
         PRESERVE_DN => 'Y',
         CAKEY       => $main->{'CA'}->{$ca}->{'dir'}."/cacert.key",
         CACERT      => $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem",
         DAYS        => $days,
         PASSWD      => $passwd,
         CA_NAME     => $certtype."_ca"
         );

   $certout  = $main->{'CA'}->{$ca}->{'dir'}."/newcerts/".$serial.".pem";
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$req.".pem";
   if (not -s $certout) {
         $main->print_warning("issuCert failed");
         return;
   }
   open(IN, "<$certout") || do {
      $main->print_warning("can't open certout");
      return;
   };
   open(OUT, ">$certfile") || do {
      $main->print_warning("can't open certfile");
      return;
   };
   print OUT while(<IN>);
   close IN; close OUT;

   $action = $main->show_req_del_question();

   if($action eq "Yes") {
      unlink($reqfile);
   }

   $main->{'CERT'}->parse_cert(
         $main,
         $req
         );

   $main->create_mframe();
     
   return;
}

sub import_req {
   my $self = shift;
   my $main = shift;

   my($reqname, $reqfile, $action, $infile, $parsed, $req, $ca,
      $out);

   $ca = $main->{'CA'}->{'actca'};

   $action = $main->show_req_import_dialog( 
         \$infile,
         ); 

   return if $action eq "Cancel";
   return if(not defined($infile));

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $req = OpenCA::REQ->new(
         SHELL  => $main->{'OpenSSL'},
         INFILE => $infile,
         FORMAT => "PEM"
         );

   if(not defined($req)) {
      $req = OpenCA::REQ->new(
      SHELL  => $main->{'OpenSSL'},
      INFILE => $infile,
      FORMAT => "DER"
      );
   }

   if(not defined($req)) {
      $main->print_warning("Can't read Request");
      return;
   }

   $parsed = $req->getParsed();

   if(not defined($parsed)) {
      $main->print_warning("Can't read Request");
      return;
   }

   # these x.501 attributes are not scanned within OpenCA::REQ
   if($parsed->{'DN'} =~ /[\,\/\s]ST=([^\,\/\n]+)/i) {
      $parsed->{'ST'} = $1;
   }
   if($parsed->{'DN'} =~ /[\,\/\s]N=([^\,\/\n]+)/i) {
      $parsed->{'N'} = $1;
   }
   if($parsed->{'DN'} =~ /[\,\/\s]G=([^\,\/\n]+)/i) {
      $parsed->{'G'} = $1;
   }
   if($parsed->{'DN'} =~ /[\,\/\s]I=([^\,\/\n]+)/i) {
      $parsed->{'I'} = $1;
   }
   if($parsed->{'DN'} =~ /[\,\/\s]T=([^\,\/\n]+)/i) {
      $parsed->{'T'} = $1;
   }
   if($parsed->{'DN'} =~ /[\,\/\s]D=([^\,\/\n]+)/i) {
      $parsed->{'D'} = $1;
   }

   $action = $main->show_req_import_verification(
         $parsed
         );

   return if $action eq 'Cancel';

   $reqname = MIME::Base64::encode(
         $parsed->{'CN'}.":".
         $parsed->{'EMAIL'}.":".
         $parsed->{'OU'}->[0].":".
         $parsed->{'O'}.":".
         $parsed->{'L'}.":".
         $parsed->{'ST'}.":".
         $parsed->{'C'},
         ''
         );

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$reqname.".pem";

   $out = $req->getPEM();

   open(OUT, ">$reqfile") || do {
      $main->print_warning("Can't open outfile");
      return;
   };
   print OUT $out;
   close OUT;

   $self->{$reqname}->{'parsed'} = $parsed;

   $self->{$reqname}->{'txt'} = $req->getTXT();
   
   $main->create_mframe();

   return;
}

sub parse_req {
   my $self = shift;
   my $main = shift;
   my $name = shift;
   
   my ($parsed, $ca, $reqfile, $req);

   $ca = $main->{'CA'}->{'actca'};

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $reqfile = $main->{'CA'}->{$ca}->{'dir'}."/req/".$name.".pem";

   $req = OpenCA::REQ->new(
         SHELL  => $main->{'OpenSSL'},
         INFILE => $reqfile,
         FORMAT => "PEM"
         );

   $self->{$name}->{'parsed'} = $req->getParsed();

   # these x.501 attributes are not scanned within OpenCA::REQ
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]ST=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'ST'} = $1;
   }
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]N=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'N'} = $1;
   }
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]G=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'G'} = $1;
   }
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]I=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'I'} = $1;
   }
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]T=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'T'} = $1;
   }
   if($self->{$name}->{'parsed'}->{'DN'} =~ /[\,\/\s]D=([^\,\/\n]+)/i) {
      $self->{$name}->{'parsed'}->{'D'} = $1;
   }

   $self->{$name}->{'txt'} = $req->getTXT();
}

1
